@@ -265,40 +265,47 @@ H5Dread <- function( h5dataset, h5spaceFile=NULL, h5spaceMem=NULL, buf = NULL, c
 #' @param h5dataset Object of class [H5IdComponent-class] representing an open HDF5 
 #' dataset.
 #' @param buf The R object containing the data to be written to the dataset.
+#' @param h5type Datatype of the HDF5 dataset to be written.  If left as `NULL`
+#' it will use the dataype of the R object supplied to `buf`.
 #' @param h5spaceMem,h5spaceFile [H5IdComponent-class] objects representing the 
 #' memory and file dataspaces respectively.  If these are left `NULL` dataspaces 
 #' that match the size and shape of `h5dataset` will be used.
 #' 
 #' @export
-H5Dwrite <- function( h5dataset, buf, h5spaceMem=NULL, h5spaceFile=NULL ) {
+H5Dwrite <- function( h5dataset, buf, h5type=NULL, h5spaceMem=NULL, h5spaceFile=NULL ) {
   h5checktype(h5dataset, "dataset")
   h5checktypeOrNULL(h5spaceFile, "dataspace")
   h5checktypeOrNULL(h5spaceMem, "dataspace")
+  if (is.null(h5type)) { tidMem <- NULL } else { tidMem <- h5type }
   if (is.null(h5spaceMem)) { sidMem <- NULL } else { sidMem <- h5spaceMem@ID }
   if (is.null(h5spaceFile)) { sidFile <- NULL } else { sidFile <- h5spaceFile@ID }
-  invisible(.Call("_H5Dwrite", h5dataset@ID, buf, sidFile, sidMem, h5dataset@native, PACKAGE='rhdf5'))
+  invisible(.Call("_H5Dwrite", h5dataset@ID, buf, sidFile, sidMem, h5type, h5dataset@native, PACKAGE='rhdf5'))
 }
 
 #' Change the dimensions of an HDF5 dataset
-#' 
-#' @details This function can only be applied to datasets that meet the 
-#' following criteria:
+#'
+#' @details This function can only be applied to datasets that meet the
+#'   following criteria:
 #' * A chunked dataset with unlimited dimensions
-#' * A chunked dataset with fixed dimensions if the new dimension sizes are 
-#' less than the maximum sizes set with maxdims
-#' #' 
-#' @param h5dataset Object of class [H5IdComponent-class] representing an open HDF5 
-#' dataset.
+#' * A chunked dataset with fixed dimensions if the new dimension sizes are
+#'   less than the maximum sizes set with maxdims
+#'
+#' @param h5dataset Object of class [H5IdComponent-class] representing an open
+#'   HDF5 dataset.
 #' @param size An integer vector with the new dimension of the dataset.
-#' 
+#'
+#' @returns A logical vector of length 1.  Value will be `TRUE` if the operation
+#'   was sucessful and `FALSE` otherwise.
+#'
 #' @author Bernd Fischer, Mike Smith
-#' 
+#'
 #' @export
 H5Dset_extent <- function( h5dataset, size) {
     h5checktype(h5dataset, "dataset")
     size <- as.numeric(size)
     if (!h5dataset@native) size <- rev(size)
-    invisible(.Call("_H5Dset_extent", h5dataset@ID, size, PACKAGE='rhdf5'))
+    res <- .Call("_H5Dset_extent", h5dataset@ID, size, PACKAGE='rhdf5')
+    return(invisible(res >= 0))
 }
 
     
\ No newline at end of file
