@@ -298,6 +298,27 @@ SEXP H5Aread_helper_REFERENCE(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hi
   return Rval;
 }
 
+SEXP H5Aread_helper_ENUM(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t dtype_id) {
+  
+  SEXP Rval = PROTECT(allocVector(STRSXP, (int) n));
+  
+  void *buf = R_alloc(H5Tget_size(dtype_id), n);
+  H5Aread(attr_id, dtype_id, buf);
+  
+  size_t max_string_length = 1024;
+  char *st = H5allocate_memory(max_string_length, FALSE);
+  for (int i=0; i < n; i++) {
+    memset(st, 0, max_string_length);
+    H5Tenum_nameof	(	dtype_id, buf, st, max_string_length);
+    SET_STRING_ELT(Rval, i, mkChar(st));
+    buf += H5Tget_size(dtype_id);
+  }
+  H5free_memory(st);
+  
+  UNPROTECT(1);
+  return Rval;
+}
+
 SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, int bit64conversion ) {
     
     hid_t dtype_id;
@@ -318,6 +339,9 @@ SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, int bit64con
   case H5T_REFERENCE: {
     Rval = H5Aread_helper_REFERENCE(attr_id, n, Rdim, _buf, dtype_id);
   } break;
+  case H5T_ENUM: {
+    Rval = H5Aread_helper_ENUM(attr_id, n, Rdim, _buf, dtype_id);
+  } break;
   case H5T_COMPOUND:
  /* { */
  /*    Rval = H5Aread_helper_COMPOUND(attr_id, n, Rdim, _buf, dtype_id); */
@@ -325,7 +349,6 @@ SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, int bit64con
   case H5T_TIME:
   case H5T_BITFIELD:
   case H5T_OPAQUE:
-  case H5T_ENUM:
   case H5T_VLEN:
   case H5T_ARRAY:
   default: {
@@ -418,28 +441,7 @@ SEXP _H5Awrite( SEXP _attr_id, SEXP _buf) {
         break;
     case STRSXP :
         mem_type_id = H5Aget_type(attr_id);
-        size_t stsize;
-        if (H5Tis_variable_str(mem_type_id)) {
-            const char ** strbuf = (const char **)R_alloc(LENGTH(_buf),sizeof(const char*));
-            for (int i=0; i < LENGTH(_buf); i++) {
-            strbuf[i] = CHAR(STRING_ELT(_buf,i));
-            }
-            buf = strbuf;
-        } else {
-            stsize = H5Tget_size( mem_type_id );
-            char * strbuf = (char *)R_alloc(LENGTH(_buf),stsize);
-            int z=0;
-            int j;
-            for (int i=0; i < LENGTH(_buf); i++) {
-            for (j=0; (j < LENGTH(STRING_ELT(_buf,i))) & (j < (stsize-1)); j++) {
-                strbuf[z++] = CHAR(STRING_ELT(_buf,i))[j];
-            }
-            for (; j < stsize; j++) {
-                strbuf[z++] = '\0';
-            }
-            }
-            buf = strbuf;
-        }
+        buf = read_string_datatype(mem_type_id, _buf);
         break;
     case S4SXP : 
       if(R_check_class_etc(_buf, H5Ref) >= 0) {
