@@ -117,6 +117,7 @@ static R_CallMethodDef libraryRCalls[] = {
   {"_H5Scombine_hyperslab", (DL_FUNC) &_H5Scombine_hyperslab, 6},
   {"_H5Scombine_select", (DL_FUNC) &_H5Scombine_select, 3},
   {"_H5Sselect_index", (DL_FUNC) &_H5Sselect_index, 3},
+  {"_H5Tget_class", (DL_FUNC) &_H5Tget_class, 1},
   {"_H5Tcopy", (DL_FUNC) &_H5Tcopy, 1},
   {"_H5Tset_size", (DL_FUNC) &_H5Tset_size, 2},
   {"_H5Tget_size", (DL_FUNC) &_H5Tget_size, 1},
@@ -329,6 +330,8 @@ static R_CallMethodDef libraryRCalls[] = {
   {"_h5errorHandling", (DL_FUNC) &_h5errorHandling, 1},
   {"_h5fileLock", (DL_FUNC) &_h5fileLock, 1},
   {"_h5listOpenObjects", (DL_FUNC) &_h5listOpenObjects, 1},
+  {"_h5getEnumNames", (DL_FUNC) &_h5getEnumNames, 1},
+  {"_h5getEnumValues", (DL_FUNC) &_h5getEnumValues, 1},
 #ifdef _H5P_filters
   {"_H5Pset_lzf", (DL_FUNC) &_H5Pset_lzf, 2},
   {"_H5Pset_bzip2", (DL_FUNC) &_H5Pset_bzip2, 2},
